;; -*- Lisp -*-
;; $Id: $
;; absolute-darkness/theme.jl

;; this is from absolue S :

;; Copyright (C) 1999 Mike Bond <mbond@erols.com>

;; This file is part of sawmill.

;; sawmill is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; sawmill is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with sawmill; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;; The basic bar images and design of this theme are from Hallvar Helleseth's
;; Absolute E theme for Enlightenment. The original theme.jl is from
;; John Harper's absolute-e theme for sawmill.

;; Note: The button and foreground images were used along with gimp and it's
;;	 layers to create the close, iconify and maximize buttons. I've kept
;;	 the originals around as someone may want to play with the buttons
;;	 some more. In particular, I'm still not real happy with the close
;;	 button, I tried an x and it was horrible, other variants weren't so
;;	 bad but this is the one I've liked the most so far.
;;
;; sawmill --batch compiler -f compile-batch theme.jl

;; from the ABOUT file:

;;    Thanks for using Absolute E, the theme that thinks diffrently.
;;    This theme is created by Hallvar Helleseth using nothing but The
;;    Gimp.  Everything is originaly made by me - Propably the only
;;    Etheme that can say that ;) Though alot of people have helped me
;;    (too many to list here/can't remeber all of them either...) But I
;;    give them a BIG thank you!

;;    There is a GTK and WMaker version of this theme available at
;;    themes.org

;;    This theme is going to be following the cvs version of E and not
;;    the stable version (DR0.15.5)

;;    Reach me at hallvar@ii.uib.no (www.ii.uib.no/~hallvar)

;;    Date: July 19 1999.

;; preferences

(defgroup absolute-darkness-frame "Absolute Darkness")

(defcustom absolute-darkness:foreground "white"
  "Title foreground color"
  :group absolute-darkness-frame
  :type color)

(defcustom absolute-darkness:full-title nil
  "Use full width of window for title and buttons"
  :group absolute-darkness-frame
  :type boolean)

(defcustom absolute-darkness:shrink-shaped-title nil
  "Shrink title to width of title text when shaded"
  :group absolute-darkness-frame
  :type boolean)

(defcustom absolute-darkness:border-dimension 4
  "Width of window border"
  :group absolute-darkness-frame
  :type number
  :range (1 . nil))

(let*
    ((title-width (lambda (w)
		    (let
			((w-width (car (window-dimensions w))))
		      (cond ((eq absolute-darkness:full-title 'nil)
			     (max 0 (min (- w-width 64) (max (- (/ w-width 2) 64) (+ 10 (text-width (window-name w)))))))
			    (t
			     (max 0 (- w-width 64)))))))

     (title-width-shaped (lambda (w)
			   (let
			       ((w-width (car (window-dimensions w))))
			     (cond ((eq absolute-darkness:shrink-shaped-title 'nil)
				    (title-width w))
				   (t
				    (max 0 (min (- w-width 64) (+ 10 (text-width (window-name w))))))))))

     (border-dimension (lambda () absolute-darkness:border-dimension))
     (-border-dimension (lambda () (- absolute-darkness:border-dimension)))
     (foreground () absolute-darkness:foreground)

     ;; called after the WM_NAME property changes
     (update-text-width
      (lambda (w)
	(if (eq (window-get w 'current-frame-style) 'absolute-darkness)
	    (rebuild-frame w))))

     (rebuild
      (lambda()
	(mapc (lambda (w)
		(when (eq (window-get w 'current-frame-style) 'absolute-darkness)
		  (rebuild-frame w))) (managed-windows))))

     (bar-images (mapcar (lambda (i)
			   (set-image-border i 4 4 4 4))
			 (list (make-image "bar_normal.png")
			       (make-image "bar_normal_active.png")
			       (make-image "bar_hilited_active.png")
			       (make-image "bar_clicked_active.png"))))

     (close-images (mapcar (lambda (i)
			     (set-image-border i 4 4 4 4))
			   (list (make-image "close_normal.png")
				 (make-image "close_normal_active.png")
				 (make-image "close_hilited_active.png")
				 (make-image "close_clicked_active.png"))))

     (iconify-images (mapcar (lambda (i)
			       (set-image-border i 4 4 4 4))
			     (list (make-image "iconify_normal.png")
				   (make-image "iconify_normal_active.png")
				   (make-image "iconify_hilited_active.png")
				   (make-image "iconify_clicked_active.png"))))

     (maximize-images (mapcar (lambda (i)
				(set-image-border i 4 4 4 4))
			      (list (make-image "maximize_normal.png")
				    (make-image "maximize_normal_active.png")
				    (make-image "maximize_hilited_active.png")
				    (make-image "maximize_clicked_active.png"))))

     (menu-images (mapcar (lambda (i)
                                (set-image-border i 4 4 4 4))
                              (list (make-image "menu_normal.png")
                                    (make-image "menu_normal_active.png")
                                    (make-image "menu_hilited_active.png")
                                    (make-image "menu_clicked_active.png"))))


     (frame
      `(;; menu button
	((background . ,menu-images)
	 (left-edge . 0)
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . menu-button))
	;; title bar
	((background . ,bar-images)
	 (foreground . ,foreground)
	 (text . ,window-name)
	 (x-justify . center)
	 (y-justify . center)
	 (left-edge . 16)
	 (width . ,title-width)
	 (top-edge . -16)
	 (class . title))
	;; iconify button
	((background . ,iconify-images)
	 (left-edge . ,(lambda (w) (+ (title-width w) 16)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . iconify-button))
	;; maximize button
	((background . ,maximize-images)
	 (left-edge . ,(lambda (w) (+ (title-width w) 32)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . maximize-button))
	;; close button
	((background . ,close-images)
	 (left-edge . ,(lambda (w) (+ (title-width w) 48)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . close-button))
	;; rhs bit of title
	((background . ,bar-images)
	 (left-edge . ,(lambda (w) (+ (title-width w) 64)))
	 (right-edge . 0)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-border))
	;; left frame
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . 0)
	 (bottom-edge . 0)
	 (class . right-border))
	;; right frame
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . 0)
	 (bottom-edge . 0)
	 (class . left-border))
	;; bottom frame
	((background . ,bar-images)
	 (left-edge . 0)
	 (right-edge . 0)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-border))
	;; top-left corner
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-left-corner))
	;; top-right corner
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-right-corner))
	;; bottom-left corner
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-left-corner))
	;; bottom-right corner
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-right-corner))))

     (shaped-frame
      `(;; menu button
	((background . ,menu-images)
	 (left-edge . 0)
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . menu-button))
	;; title bar
	((background . ,bar-images)
	 (foreground . ,foreground)
	 (text . ,window-name)
	 (x-justify . center)
	 (y-justify . center)
	 (left-edge . 16)
	 (width . ,title-width-shaped)
	 (top-edge . -16)
	 (class . title))
	;; iconify button
	((background . ,iconify-images)
	 (left-edge . ,(lambda (w) (+ (title-width-shaped w) 16)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . iconify-button))
	;; maximize button
	((background . ,maximize-images)
	 (left-edge . ,(lambda (w) (+ (title-width-shaped w) 32)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . maximize-button))
	;; close button
	((background . ,close-images)
	 (left-edge . ,(lambda (w) (+ (title-width-shaped w) 48)))
	 (width . 16)
	 (top-edge . -16)
	 (height . 16)
	 (class . close-button))))

     (transient-frame
      `(((background . ,bar-images)
	 (left-edge . 0)
	 (right-edge . 0)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-border))
	((background . ,bar-images)
	 (left-edge . 0)
	 (right-edge . 0)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-border))
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . 0)
	 (bottom-edge . 0)
	 (class . left-border))
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . 0)
	 (bottom-edge . 0)
	 (class . right-border))
	;; top-right corner
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-right-corner))
	;; top-left corner
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-left-corner))
	;; bottom-left corner
	((background . ,bar-images)
	 (left-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-left-corner))
	;; bottom-right corner
	((background . ,bar-images)
	 (right-edge . ,-border-dimension)
	 (width . ,border-dimension)
	 (bottom-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . bottom-right-corner))))

     (shaped-transient-frame
      `(((background . ,bar-images)
	 (left-edge . 0)
	 (right-edge . 0)
	 (top-edge . ,-border-dimension)
	 (height . ,border-dimension)
	 (class . top-border)))))

  (add-frame-style 'absolute-darkness
		   (lambda (w type)
		     (cond ((eq type 'shaped)
			    shaped-frame)
			   ((eq type 'transient)
			    transient-frame)
			   ((eq type 'shaped-transient)
			    shaped-transient-frame)
			   ((eq type 'unframed)
			    nil-frame)
			   (t
			    frame))))

  (mapc (lambda (sym)
	  (custom-set-property sym ':after-set rebuild))
	'(absolute-darkness:foreground
	  absolute-darkness:full-title
	  absolute-darkness:shrink-shaped-title
	  absolute-darkness:border-dimension))

  (call-after-property-changed 'WM_NAME update-text-width))
